/*
* JBoss, Home of Professional Open Source
* Copyright 2006, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/
package org.jboss.deployers.vfs.plugins.structure.modify;

import java.util.HashMap;
import java.util.Map;

import org.jboss.deployers.spi.structure.ModificationType;

/**
 * @author <a href="mailto:ales.justin@jboss.com">Ales Justin</a>
 */
public class ModificationActions
{
   private static final Map<ModificationType, ModificationAction> actions;

   static
   {
      actions = new HashMap<ModificationType, ModificationAction>();
      actions.put(ModificationType.UNPACK, new UnpackModificationAction());
      actions.put(ModificationType.EXPLODE, new ExplodeModificationAction());
      actions.put(ModificationType.TEMP, new TempModificationAction());
   }

   /**
    * Get the modification action.
    *
    * @param type the modification type
    * @return modification action or null if it not supported
    */
   public static ModificationAction getAction(ModificationType type)
   {
      return actions.get(type);
   }
}
